require 'base64'

module Nexaas
  module Cipher
    # Salt: represents the disturber salt for ciphering
    # :reek:BooleanParameter
    # :reek:ControlParameter
    class Salt
      def initialize(salt, base64: false)
        @salt = base64 ? Base64.decode64(salt) : salt
      end

      def salt(code = nil, base64: false, truncate: false)
        return Base64.encode64(salt(code, truncate: truncate)).rstrip if base64
        return @salt unless code
        return "\0" * code.length if @salt.empty?
        dup_salt(code, @salt, truncate)
      end

      def salt?
        !@salt.empty?
      end

      private

      # current :reek:FeatureEnvy
      # current.length :reek:DuplicateMethodCall
      def dup_salt(code, current, truncate)
        expected_length = code.length
        expected_length = [current.length, expected_length].max unless truncate
        current <<= @salt while current.length < expected_length
        current[0, expected_length]
      end
    end
  end
end
